#version 450

// ----------------------------------------------------------------------------
// Input Variables
// ----------------------------------------------------------------------------

// Task 5.2: Add a uniform variable for eye(camera) position.
layout(location = 6) uniform vec3 cam_position;

// Task 5.2: Add a uniform variable for light's position and ambient+diffuse+specular terms.
layout(location = 7) uniform vec3 light_position;
layout(location = 8)  uniform vec3 light_ambient_color;
layout(location = 9)  uniform vec3 light_diffuse_color;
layout(location = 10) uniform vec3 light_specular_color;

// Task 5.2: Add a uniform variable for material's ambient+diffuse+specular+shininess terms.
layout(location = 11) uniform vec3 model_ambient_color;
layout(location = 12) uniform vec3 model_diffuse_color;
layout(location = 13) uniform vec3 model_specular_color;
layout(location = 14) uniform float model_shininess;

// The intpolated position from the vertex shader.
layout(location = 0) in vec3 fs_position;
// The intpolated normal from the vertex shader.
// N dle wiki
layout(location = 1) in vec3 fs_normal;
// The intpolated texture coordinates from the vertex shader.
layout(location = 2) in vec2 fs_texture_coordinate;

// ----------------------------------------------------------------------------
// Output Variables
// ----------------------------------------------------------------------------

// The final output color.
layout(location = 0) out vec4 final_color;

// ----------------------------------------------------------------------------
// Main Method
// ----------------------------------------------------------------------------
void main()
{
	// Task 5.2: Compute (N)ormal, (L)ight, (E)ye, (H)alf vectors.
	//dle Wiki: N, L, V (eye, camera), H (analog k R, odrazenemu vektoru)
	vec3 N = normalize(fs_normal);
	vec3 L = normalize(light_position - fs_position);
	vec3 V = normalize(cam_position - fs_position);
	vec3 H = normalize(L + V);

	float LdotN = max( dot(L,N), 0 ); //uhel resp. magnituda pro difuzivitu
	float HdotN = max( dot(H,N), 0 ); //uhel resp. magnituda pro specularitu

	vec3 ambient_color = model_ambient_color * light_ambient_color; //wiki: i_a
	vec3 diffuse_color = LdotN * model_diffuse_color * light_diffuse_color; //wiki: Lm.N * i_d
	vec3 specular_color = pow(HdotN,model_shininess) * model_specular_color * light_specular_color; //wiki: (Rm.V)^alfa * i_s

	// Task 5.2: Compute dot products between N and L, N and H.
	// Task 5.2: Compute material values multiplied by light's values.
	// Task 5.2: Compute final color using Blinn�Phong shading model.
	// Task 5.2: Use attenuation based on the light distance for point lights.
	// Task 5.2: Differentiate between directional and point light based on its .w value in position.

	final_color = vec4(ambient_color + diffuse_color, 1.0);
}
